
#include "v3.h"
#include <math.h>
#include <float.h>

#include <iostream>

using namespace std;

V3::V3(float _x, float _y, float _z) : x(_x), y(_y), z(_z) {}

V3 V3::operator+(V3 op2) {

  V3 ret;
  ret.x = x+op2.x;
  ret.y = y+op2.y;
  ret.z = z+op2.z;
  return ret;

}

V3 V3::operator-(V3 op2) {

  V3 ret;
  ret.x = x-op2.x;
  ret.y = y-op2.y;
  ret.z = z-op2.z;
  return ret;

}

V3 V3::operator^(V3 op2) {

  V3 ret;
  ret.x = y*op2.z - z*op2.y;
  ret.y = z*op2.x - x*op2.z;
  ret.z = x*op2.y - y*op2.x;

  return ret;

}

float V3::operator*(V3 op2) {

  return (x*op2.x+y*op2.y+z*op2.z);

}

ostream& operator<<(ostream &ostr, V3 v) {

  return ostr << v.x << " " << v.y << " " << v.z;

}

float& V3::operator[](int i) {

  switch (i) {
    case 0: return x;
    case 1: return y;
    case 2: return z;
    default:
      cerr << "INTERNAL ERROR: bad index" << endl;
      return x;
  }

}

V3 V3::operator*(float sc) {

  V3 ret;
  ret.x = x*sc;
  ret.y = y*sc;
  ret.z = z*sc;
  return ret;

}

V3 V3::operator/(float sc) {

  V3 ret;
  ret.x = x/sc;
  ret.y = y/sc;
  ret.z = z/sc;
  return ret;

}

unsigned int V3::UIntColor() {

  unsigned int ret = 0xFF000000;
  ret += (unsigned int)(255.0f*x);
  ret += 256*(unsigned int)(255.0f*y);
  ret += 256*256*(unsigned int)(255.0f*z);
  return ret;

}

V3 V3::UnitVector() {

  V3 ret = (*this)/Length();
  return ret;

}

float V3::Length() {

  return sqrtf(x*x+y*y+z*z);

}

BBox::BBox() {

  corners[0] = V3(FLT_MAX, FLT_MAX, FLT_MAX);
  corners[1] = V3(-FLT_MAX, -FLT_MAX, -FLT_MAX);

}

void BBox::AddPoint(V3 P) {

  if (corners[0][0] > P[0])
    corners[0][0] = P[0];
  if (corners[0][1] > P[1])
    corners[0][1] = P[1];
  if (corners[0][2] > P[2])
    corners[0][2] = P[2];
  if (corners[1][0] < P[0])
    corners[1][0] = P[0];
  if (corners[1][1] < P[1])
    corners[1][1] = P[1];
  if (corners[1][2] < P[2])
    corners[1][2] = P[2];

}


bool BBox::Clip(BBox bbox2) {


  if (corners[0][0] < bbox2.corners[0][0])
    corners[0][0] = bbox2.corners[0][0];
  if (corners[1][0] > bbox2.corners[1][0])
    corners[1][0] = bbox2.corners[1][0];

  if (corners[0][1] < bbox2.corners[0][1])
    corners[0][1] = bbox2.corners[0][1];
  if (corners[1][1] > bbox2.corners[1][1])
    corners[1][1] = bbox2.corners[1][1];

  if (corners[0][2] < bbox2.corners[0][2])
    corners[0][2] = bbox2.corners[0][2];
  if (corners[1][2] > bbox2.corners[1][2])
    corners[1][2] = bbox2.corners[1][2];

  if (corners[1][0] < corners[0][0])
    return false;
  if (corners[1][1] < corners[0][1])
    return false;
  if (corners[1][2] < corners[0][2])
    return false;

  return true;

}

void V3::SetFromUIntColor(unsigned int col) {

  x = ((float)(((unsigned char*)&col)[0]))/255.0f;
  y = ((float)(((unsigned char*)&col)[1]))/255.0f;
  z = ((float)(((unsigned char*)&col)[2]))/255.0f;

}

